@@ -40,13 +40,6 @@ HostInstaller::HostInstaller()
     , TARGET_DIR_VIVALDI("/Library/Application Support/Vivaldi/NativeMessagingHosts")
     , TARGET_DIR_TOR_BROWSER("/Library/Application Support/TorBrowser-Data/Browser/Mozilla/NativeMessagingHosts")
     , TARGET_DIR_BRAVE("/Library/Application Support/BraveSoftware/Brave-Browser/NativeMessagingHosts")
-#elif defined(Q_OS_LINUX)
-    , TARGET_DIR_CHROME("/.config/google-chrome/NativeMessagingHosts")
-    , TARGET_DIR_CHROMIUM("/.config/chromium/NativeMessagingHosts")
-    , TARGET_DIR_FIREFOX("/.mozilla/native-messaging-hosts")
-    , TARGET_DIR_VIVALDI("/.config/vivaldi/NativeMessagingHosts")
-    , TARGET_DIR_TOR_BROWSER("/.tor-browser/app/Browser/TorBrowser/Data/Browser/.mozilla/native-messaging-hosts")
-    , TARGET_DIR_BRAVE("/.config/BraveSoftware/Brave-Browser/NativeMessagingHosts")
 #elif defined(Q_OS_WIN)
     // clang-format off
     , TARGET_DIR_CHROME("HKEY_CURRENT_USER\\Software\\Google\\Chrome\\NativeMessagingHosts\\org.keepassxc.keepassxc_browser")
@@ -56,6 +49,13 @@ HostInstaller::HostInstaller()
     , TARGET_DIR_VIVALDI(TARGET_DIR_CHROME)
     , TARGET_DIR_TOR_BROWSER(TARGET_DIR_FIREFOX)
     , TARGET_DIR_BRAVE(TARGET_DIR_CHROME)
+#else
+    , TARGET_DIR_CHROME("/.config/google-chrome/NativeMessagingHosts")
+    , TARGET_DIR_CHROMIUM("/.config/chromium/NativeMessagingHosts")
+    , TARGET_DIR_FIREFOX("/.mozilla/native-messaging-hosts")
+    , TARGET_DIR_VIVALDI("/.config/vivaldi/NativeMessagingHosts")
+    , TARGET_DIR_TOR_BROWSER("/.tor-browser/app/Browser/TorBrowser/Data/Browser/.mozilla/native-messaging-hosts")
+    , TARGET_DIR_BRAVE("/.config/BraveSoftware/Brave-Browser/NativeMessagingHosts")
 #endif
 {
 }
@@ -109,9 +109,7 @@ void HostInstaller::installBrowser(SupportedBrowsers browser,
 #ifdef Q_OS_WIN
         // Create a registry key
         QSettings settings(getTargetPath(browser), QSettings::NativeFormat);
-        if (!registryEntryFound(settings)) {
-            settings.setValue("Default", getPath(browser));
-        }
+        settings.setValue("Default", getPath(browser));
 #endif
         // Always create the script file
         QJsonObject script = constructFile(browser, proxy, location);
@@ -128,9 +126,7 @@ void HostInstaller::installBrowser(SupportedBrowsers browser,
 #ifdef Q_OS_WIN
         // Remove the registry entry
         QSettings settings(getTargetPath(browser), QSettings::NativeFormat);
-        if (registryEntryFound(settings)) {
-            settings.remove("Default");
-        }
+        settings.remove("Default");
 #endif
     }
 }
@@ -171,7 +167,7 @@ QString HostInstaller::getTargetPath(SupportedBrowsers browser) const
     case SupportedBrowsers::TOR_BROWSER:
         return TARGET_DIR_TOR_BROWSER;
     case SupportedBrowsers::BRAVE:
-      return TARGET_DIR_BRAVE;
+        return TARGET_DIR_BRAVE;
     default:
         return QString();
     }
@@ -194,11 +190,11 @@ QString HostInstaller::getBrowserName(SupportedBrowsers browser) const
     case SupportedBrowsers::FIREFOX:
         return "firefox";
     case SupportedBrowsers::VIVALDI:
-      return "vivaldi";
+        return "vivaldi";
     case SupportedBrowsers::TOR_BROWSER:
         return "tor-browser";
     case SupportedBrowsers::BRAVE:
-      return "brave";
+        return "brave";
     default:
         return QString();
     }
@@ -299,9 +295,9 @@ QJsonObject HostInstaller::constructFile(SupportedBrowsers browser, const bool&
 
     QJsonObject script;
     script["name"] = HOST_NAME;
-    script["description"] = "KeePassXC integration with native messaging support";
+    script["description"] = QString("KeePassXC integration with native messaging support");
     script["path"] = path;
-    script["type"] = "stdio";
+    script["type"] = QString("stdio");
 
     QJsonArray arr;
     if (browser == SupportedBrowsers::FIREFOX || browser == SupportedBrowsers::TOR_BROWSER) {
